/**************************************************************************************************
 * THE OMEGA LIB PROJECT
 *-------------------------------------------------------------------------------------------------
 * Copyright 2010-2015		Electronic Visualization Laboratory, University of Illinois at Chicago
 * Authors:										
 *  Alessandro Febretti		febret@gmail.com
 *-------------------------------------------------------------------------------------------------
 * Copyright (c) 2010-2015, Electronic Visualization Laboratory, University of Illinois at Chicago
 * All rights reserved.
 * Redistribution and use in source and binary forms, with or without modification, are permitted 
 * provided that the following conditions are met:
 * 
 * Redistributions of source code must retain the above copyright notice, this list of conditions 
 * and the following disclaimer. Redistributions in binary form must reproduce the above copyright 
 * notice, this list of conditions and the following disclaimer in the documentation and/or other 
 * materials provided with the distribution. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR 
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND 
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR 
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE  GOODS OR SERVICES; LOSS OF 
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
 * ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *************************************************************************************************/
#ifndef __TEXTURE_SOURCE_H__
#define __TEXTURE_SOURCE_H__

#include "osystem.h"
#include "omega/Texture.h"

namespace omega {
	struct DrawContext;

	///////////////////////////////////////////////////////////////////////////////////////////////
	class OMEGA_API TextureSource: public ReferenceType
	{
	public:
		TextureSource(): 
			myTextureUpdateFlags(0), 
			myRequireExplicitClean(false) { memset(myDirty, 0, sizeof(myDirty)); }
		virtual ~TextureSource() {}

		virtual Texture* getTexture(const DrawContext& context);
		virtual void attachTexture(Texture* tex, const DrawContext& context);

		virtual bool isDirty() { return myDirty; }
		virtual void setDirty(bool value = true);

		//! When enabled, the TextureSource object stays dirty even after all 
		//! the associated Textures have been updated, and will be marked as 
		//! clean only Through an explicit setDirty(false) call.
		void requireExplicitClean(bool value) { myRequireExplicitClean = value; }

	protected:
		virtual void refreshTexture(Texture* texture, const DrawContext& context) = 0;

	private:
		Ref<Texture> myTextures[GpuContext::MaxContexts];
		uint64_t myTextureUpdateFlags;
		bool myRequireExplicitClean;
		bool myDirty[GpuContext::MaxContexts];
	};
}; // namespace omega

#endif